
jQuery(document).ready(function($) {
    class BetSlipManager {
        constructor(config) {
            this.config = config;
            this.betSlip = [];
            this.initEventListeners();
        }

        initEventListeners() {
            $('#loginButton').on('click', this.handleLogin.bind(this));
            $('#sportSelect').on('change', this.handleSportChange.bind(this));
            $('body').on('change', '#categorySelect', this.handleCategoryChange.bind(this));
            $('body').on('change', '#typeSelect', this.handleTypeChange.bind(this));
            $('#addParlayButton').on('click', this.displayParlay.bind(this));
            $('#uploadButton').on('click', this.uploadBets.bind(this));
        }

        hideForm(formId) {
            $('#' + formId).hide().empty();
        }

        handleLogin() {
            const screenName = $('#screenName').val().trim();
            if (screenName) {
                this.screenName = screenName;
                $('#loginForm').hide();
                $('#bet-form').show();
            } else {
                alert('Please enter a screen name.');
            }
        }

        handleSportChange(event) {
            const sport = $(event.target).val();
            this.hideForm('categoryForm');
            this.hideForm('typeForm');
            this.hideForm('detailsForm');
            if (sport) {
                this.updateCategoryForm(sport);
            }
        }

        handleCategoryChange(event) {
    const category = $(event.target).val();
    const sport = $('#sportSelect').val();
    this.hideForm('typeForm');
    this.hideForm('detailsForm');
    if (category) {
        this.updateTypeForm(sport, category);
        // Correctly placed check for fetching team names
        if (sport === "Baseball" && category === "Game Outcomes") {
            this.fetchAndSetupTeamNames();
        }
        if (sport === "Baseball" && category === "Player Lines") {
            this.fetchAndSetupPlayerNames();
        }
    }
}

        

        handleTypeChange(event) {
            const type = $(event.target).val();
            const sport = $('#sportSelect').val();
            const category = $('#categorySelect').val();
            this.hideForm('detailsForm');
            if (type) {
                this.showDetailsForm(sport, category, type);
            }
        }

        updateCategoryForm(sport) {
            const categoryLabel = $('<label/>', {
                 'for': 'categorySelect',
                text: 'Select Category:',
                class: 'form-label' // Add any additional classes for styling
            });
            
            const categorySelect = $('<select></select>', { id: 'categorySelect' }).addClass('form-select');
            
            categorySelect.append(this.createOption('Select a Category', ''));
            $.each(this.config[sport].categories, (category) => {
                categorySelect.append(this.createOption(category, category));
            });
           $('#categoryForm').empty().append(categoryLabel).append(categorySelect).show();
            
        }
        
      fetchAndSetupTeamNames() {
    $.ajax({
        type: 'POST',
        url: betSlipManagerParams.ajaxurl,
        data: {
            'action': 'get_sd46_teamnames'
        },
        success: (response) => {
            if (response.success) {
                this.setupAutocompleteForWinningTeam(response.data);
            } else {
                console.error('Failed to load team names.');
            }
        },
        error: (jqXHR, textStatus, errorThrown) => {
            console.error('Error fetching team names:', textStatus, errorThrown);
        }
    });
}



setupAutocompleteForWinningTeam(teamnames) {
    const winningTeamInput = $('#winningTeam');
    // Extract the team names into an array for the autocomplete source.
    const teamNamesArray = teamnames.map(team => team.teamname);
    
    // Initialize the autocomplete feature on the input element.
    winningTeamInput.autocomplete({
        source: teamNamesArray,
        minLength: 0 // Adjust as needed for when the autocomplete should start showing suggestions.
    }).focus(function () {
        // Show the autocomplete options upon focusing the input field.
        $(this).autocomplete("search", "");
    });
}



        updateTypeForm(sport, category) {
            const typeLabel = $('<label/>', {
                'for': 'typeSelect',
                text: 'Select Type:',
                class: 'form-label' // Add any additional classes for styling
            });
    
            const typeSelect = $('<select></select>', { id: 'typeSelect' }).addClass('form-select');
            typeSelect.append(this.createOption('Select a Type', ''));
            $.each(this.config[sport].categories[category], (type) => {
                typeSelect.append(this.createOption(type, type));
            });
            
            $('#typeForm').empty().append(typeLabel).append(typeSelect).show();
            
        
        }
        

    fetchAndSetupPlayerNames() {
    $.ajax({
        type: 'POST',
        url: betSlipManagerParams.ajaxurl,
        data: {
            'action': 'get_sd46_playernames'
        },
        success: (response) => {
            if (response.success) {
                this.setupAutocompleteForPlayerName(response.data);
            } else {
                console.error('Failed to load team names.');
            }
        },
        error: (jqXHR, textStatus, errorThrown) => {
            console.error('Error fetching team names:', textStatus, errorThrown);
        }
    });
}
setupAutocompleteForPlayerName(playernames) {
    const playerNameInput = $('#playerName'); // Ensure you have an input with id="playerName"
    // Extract the player names into an array for the autocomplete source.
    const playerNamesArray = playernames.map(player => player.playername);
    
    // Initialize the autocomplete feature on the input element.
    playerNameInput.autocomplete({
        source: playerNamesArray,
        minLength: 0 // Adjust as needed for when the autocomplete should start showing suggestions.
    }).focus(function () {
        // Show the autocomplete options upon focusing the input field.
        $(this).autocomplete("search", "");
    });
}


 showDetailsForm(sport, category, type) {
    const detailsForm = $('#detailsForm');
    detailsForm.empty();

    const details = this.config[sport].categories[category][type].details;
    $.each(details, (index, detail) => {
        const label = $('<label></label>').text(detail.label).addClass('form-label');
        let inputElement;

        if (detail.type === 'select') {
         inputElement = $('<select></select>', { id: detail.id, name: detail.name });
    $.each(detail.options, (index, optionValue) => {
        inputElement.append(this.createOption(optionValue, optionValue));
    });
} else if (detail.id === "winningTeam") {
    // Handle the winningTeam as an autocomplete text input
    inputElement = $('<input>', { 
        type: 'text', 
        id: 'winningTeam', 
        name: 'winningTeam', 
        class: 'ui-autocomplete-input', 
        placeholder: detail.placeholder
    });
    // Intentionally removed the misplaced else-if block
} else if (detail.id === "playerName") {
    // Handle the playerName as an autocomplete text input
    inputElement = $('<input>', { 
        type: 'text', 
        id: 'playerName', 
        name: 'playerName', 
        class: 'ui-autocomplete-input', 
        placeholder: detail.placeholder
    });
    // The autocomplete setup will be called after the element is appended to the form
} else {
    // For other types of inputs that are not selects or the winningTeam field
    inputElement = $('<input>', { 
        type: detail.type, 
        id: detail.id, 
        name: detail.name, 
        placeholder: detail.placeholder 
    });
}
        detailsForm.append(label).append(inputElement);
        
        // Setup the autocomplete if it's the winningTeam field
        if (detail.id === "winningTeam") {
            this.fetchAndSetupTeamNames();
        }
        if (detail.id === "playerName") {
            this.fetchAndSetupPlayerNames();
        }
    });

    detailsForm.show();
    
    this.showBettingForm();
   
}


showBettingForm() {
    const bettingFormContainer = $('#bettingFormContainer'); // Append the new forms to the existing detailsForm container for simplicity
    bettingFormContainer.empty();
    // Bet Entry Form
    const betEntryLabel = $('<label/>', {
        'for': 'betEntry',
        text: 'Wager ($):',
        class: 'form-label bet-entry-label'
    });
    
        const dollarSignLabel = $('<span/>', {
        text: '$',
        class: 'dollar-sign'
    });

    const betEntryInput = $('<input/>', {
        id: 'betEntry',
        type: 'text',
        class: 'form-select bet-entry-input',
        placeholder: ' '
    });

    // Odds Form
    const oddsLabel = $('<label/>', {
        'for': 'odds',
        text: 'Bet Odds (+/-):',
        class: 'form-label'
    });

    const oddsInput = $('<input/>', {
        id: 'odds',
        type: 'text',
        class: 'form-select odds-input',
        placeholder: ' '
    });

    // Append the new forms to the bettingFormContainer
    bettingFormContainer.append(betEntryLabel, betEntryInput, oddsLabel, oddsInput);
    
     // Add to Bet Slip button
    const addButton = $('<button></button>', {
         text: 'Add to Bet Slip',
        type: 'button',
     click: () => {
        this.addToBet(false); 
    }
    }).addClass('add-button');

    bettingFormContainer.append(addButton);
    bettingFormContainer.show();
   

}

addToBet(isParlay) {
    const sport = $('#sportSelect').val();
    const category = $('#categorySelect').val();
    const type = $('#typeSelect').val();

    // Initialize an empty object to store details
    const details = {};
    // Iterate over each input in the details form
    $('#detailsForm :input').each(function() {
        const input = $(this);
        if (input.attr('type') !== 'button') { // Ignore button inputs
            details[input.attr('id')] = input.val();
        }
    });
    
    
    
    
    
    const betEntry = parseFloat($('#betEntry').val());
    const odds = parseFloat($('#odds').val());
    let betWinnings;

    // Ensure odds and betEntry are numbers and calculate betWinnings accordingly
    if (!isNaN(odds) && odds > 0 && !isNaN(betEntry)) {
        if (odds > 0) {
            betWinnings = betEntry * (odds / 100);
        } else {
            betWinnings = betEntry * (100 / Math.abs(odds)); // Assuming odds could be negative, use Math.abs() to avoid NaN
        }
    } else {
        // Handle invalid input case, set betWinnings to 0 or another default value
        betWinnings = betEntry * (100 / Math.abs(odds)); // Default or error value
    }

    details['betEntry'] = betEntry.toFixed(2);
    details['betWinnings'] = betWinnings.toFixed(2);

    const displayTextFormat = this.config[sport].categories[category][type].displayText;
    let displayText = displayTextFormat;
    for (const detailKey in details) {
        displayText = displayText.replace(`{${detailKey}}`, details[detailKey]);
    }

    this.betSlip.push({ sport, category, type, details, isParlay, displayText });

    this.displayBet();
    if (isParlay) {
        $('#addParlayButton').show();
    }
}

       displayBet() {
    $('#betOutput').empty();
    this.betSlip.forEach((bet, index) => {
        // Skip display for parlays here; they are handled separately.
        if (!bet.isParlay) {
            // Locate the correct displayText format.
            const displayTextFormat = this.config[bet.sport].categories[bet.category][bet.type].displayText;

            // Dynamically construct the display text by replacing placeholders.
            let displayText = displayTextFormat;
            for (const detailKey in bet.details) {
                displayText = displayText.replace(`{${detailKey}}`, bet.details[detailKey]);
            }

            // Create the display element.
            const betDisplay = $('<p></p>').addClass('display-bet-text').text(displayText);
            const removeBtn = $('<button></button>', {
                text: 'Remove',
                click: () => {
                    this.betSlip.splice(index, 1); // Remove this bet from the slip.
                    this.displayBet(); // Refresh the display.
                }
            }).addClass('remove-button');
    
            $('#betOutput').append(betDisplay.append(removeBtn));
        }
    });
}


        displayParlay() {
            $('#parlayOutput').empty(); // Clear existing content
            this.betSlip.forEach((bet) => {
                if (bet.isParlay) {
                    const parlayDisplay = $('<p></p>').text(`Parlay Bet on ${bet.sport}, ${bet.category}, ${bet.type}`);
                    $('#parlayOutput').append(parlayDisplay);
                }
            });
            if (this.betSlip.some(bet => bet.isParlay)) {
                $('#parlayOutput').show();
            } else {
                $('#parlayOutput').hide();
            }
        }
        
        // Hypothetical function that adds a bet to your array

          uploadBets() {
    const data = {
        action: 'upload_bets',
        bets: JSON.stringify(this.betSlip), // corrected from yourBetsArray to this.betSlip
        screenName: this.screenName
    };

    $.ajax({
        type: 'POST',
        url: betSlipManagerParams.ajaxurl,
        data: data, // use the data object we've just defined
        success: function(response) {
            console.log('Success:', response);
            alert('Bet slip has been successfully uploaded to the display.');
        },
        error: function(xhr, status, error) {
            console.error('Error:', error);
            alert('Failed to upload bet slip.');
        }
    });
}





        hideForm(formId) {
            $('#' + formId).hide().empty();
        }

        createOption(text, value) {
            return $('<option></option>').val(value).text(text);
        }

        replaceFormContent(formId, element) {
            $('#' + formId).empty().append(element).show();
        }
    }


    const config = {
    Baseball: {
        categories: {
            "Game Outcomes": {
                "Moneyline": {
                    details: [
                        {id: "winningTeam", label: "Winning Team:", type: "text", placeholder: " "},
                    ],
                    displayText: "{winningTeam} Moneyline   -   ${betEntry} to win ${betWinnings}"
                },
                "Spread": {
                    details: [
                        {id: "winningTeam", label: "Winning Team:", type: "text", placeholder: " "},
                        {id: "spread", label: "Spread:", type: "select",
                            options: [-6.5, -5.5, -4.5, -3.5, -2.5, -1.5, "+1.5", "+2.5", "+3.5", "+4.5", "+5.5", "+6.5"], placeholder: " "
                            
                        },
                    ],
                    displayText: "{winningTeam} {spread}   -   ${betEntry} to win ${betWinnings}"
                },
            },
            "Player Lines": {
                "Hits": {
                    details: [
                        {id: "playerName", label: "Player Name:", type: "text", placeholder: " "},
                        {id: "overUnder", label: "Over/Under:", type: "select", options: ["Over", "Under"]},
                        {id: "hits", label: "Hits:", type: "select", 
                        options: [0.5, 1.5, 2.5, 3.5 ], placeholder: " "},
                    ],
                    displayText: "{playerName}: {overUnder} {hits} Hits   -   ${betEntry} to win ${betWinnings}"
                },
                
                "RBIs": {
                    details: [
                        {id: "playerName", label: "Player Name:", type: "text", placeholder: " "},
                        {id: "overUnder", label: "Over/Under:", type: "select", options: ["Over", "Under"]},
                        {id: "RBIs", label: "RBIs:", type: "select", 
                        options: [0.5, 1.5, 2.5, 3.5], placeholder: " "},
                    ],
                    displayText: "{playerName}: {overUnder} {RBIs} RBIs   -   ${betEntry} to win ${betWinnings}"
                },
                
                
                "Home Runs": {
                    details: [
                        {id: "playerName", label: "Player Name:", type: "text", placeholder: " "},
                        {id: "overUnder", label: "Over/Under:", type: "select", options: ["Over", "Under"]},
                        {id: "homeRuns", label: "Home Runs:", type: "select", options: [0.5, 1.5], placeholder: " "},
                    ],
                    displayText: "{playerName}: {overUnder} {homeRuns} Home Runs   -   ${betEntry} to win ${betWinnings}"
                },
                
                "Walks": {
                    details: [
                        {id: "playerName", label: "Player Name:", type: "text", placeholder: " "},
                        {id: "overUnder", label: "Over/Under:", type: "select", options: ["Over", "Under"]},
                        {id: "walks", label: "Walks:", type: "select",
                        options: [1.5, 2.5, 3.5, 4.5], placeholder: " "},
                    ],
                    displayText: "{playerName}: {overUnder} {walks} Walks   -   ${betEntry} to win ${betWinnings}"
                },
                
                
                "Strikeouts": {
                    details: [
                        {id: "playerName", label: "Player Name:", type: "text", placeholder: " "},
                        {id: "overUnder", label: "Over/Under:", type: "select", options: ["Over", "Under"]},
                        {id: "Strikeouts", label: "Strikeouts:", type: "select", options: [3.5, 4.5, 5.5, 6.5, 7.5, 8.5, 9.5, 10.5, 11.5],placeholder: " "},
                    ],
                    displayText: "{playerName}: {overUnder} {Strikeouts} Strikeouts   -   ${betEntry} to win ${betWinnings}"
                },
            },
        },
    },
};
    const betSlipManager = new BetSlipManager(config);
    
});
