<?php
/**
 * Twenty Twenty-Four functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package Twenty Twenty-Four
 * @since Twenty Twenty-Four 1.0
 */
 
 update_post_meta(58, '_wp_page_template', 'template-bet-slip.php');
  update_post_meta(60, '_wp_page_template', 'template-view-bet.php');
 
 
function enqueue_custom_scripts() {
    wp_enqueue_script('bet-slip-manager', get_template_directory_uri() . '/js/bet-slip-manager.js', array('jquery'), '1.0', true);
    wp_localize_script('bet-slip-manager', 'betSlipManagerParams', array(
        'ajaxurl' => admin_url('admin-ajax.php')
    ));
    wp_enqueue_script('jquery-ui-autocomplete');
}
add_action('wp_enqueue_scripts', 'enqueue_custom_scripts');

function theme_enqueue_styles() {
    // Enqueue the main stylesheet
    wp_enqueue_style('main-styles', get_stylesheet_uri()); // This is your style.css

    // Enqueue custom header styles
    wp_enqueue_style('custom-header-styles', get_template_directory_uri() . '/header.css', array('main-styles'), '1.0.0', 'all');
    // The 'array('main-styles')' part sets 'style.css' as a dependency, ensuring it loads first
}

add_action('wp_enqueue_scripts', 'theme_enqueue_styles');


add_action('wp_ajax_get_sd46_teamnames', 'get_sd46_teamnames');
add_action('wp_ajax_nopriv_get_sd46_teamnames', 'get_sd46_teamnames');

add_action('wp_ajax_upload_bets', 'handle_upload_bets');
add_action('wp_ajax_nopriv_upload_bets', 'handle_upload_bets'); // Optional: remove if you don't want to handle for non-logged users

function get_sd46_teamnames() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'teamnames'; // This assumes your table name is prefixed with the WP prefix
    $query = "SELECT DISTINCT teamname FROM {$table_name}";
    $teamnames = $wpdb->get_results($query);

    if (!empty($teamnames)) {
        wp_send_json_success($teamnames);
    } else {
        wp_send_json_error('No team names found.');
    }

    wp_die();
}

add_action('wp_ajax_get_sd46_playernames', 'get_sd46_playernames');
add_action('wp_ajax_nopriv_get_sd46_playernames', 'get_sd46_playernames');

function get_sd46_playernames() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'playernames'; // Adjust the table name as needed
    $query = "SELECT DISTINCT playername FROM {$table_name}";
    $playernames = $wpdb->get_results($query);

    if (!empty($playernames)) {
        wp_send_json_success($playernames);
    } else {
        wp_send_json_error('No player names found.');
    }

    wp_die(); // Ensure you end your execution here
}

// Hook for logged-in users
add_action('wp_ajax_get_sd46_teamnames', 'get_sd46_teamnames');
// Hook for guests
add_action('wp_ajax_nopriv_get_sd46_teamnames', 'get_sd46_teamnames');


function theme_setup() {
    // Register navigation menu
    register_nav_menus(array(
        'header-menu' => 'Header Menu'
    ));
}
add_action('init', 'theme_setup');

function register_theme_menus() {
    register_nav_menus(array(
        'footer-menu' => __('Footer Menu', 'text_domain'), // 'text_domain' should be replaced with your theme's text domain
    ));
}
add_action('init', 'register_theme_menus');

function handle_upload_bets() {
    
    error_log(print_r($_POST, true)); // This will log the entire POST array to your debug.log

    $bets = json_decode(stripslashes($_POST['bets']), true);
    if (empty($bets) || !is_array($bets)) {
        wp_send_json_error('Invalid bets data provided.');
        wp_die();
    }
    global $wpdb; // Access to the WordPress database object

    // Ensure you perform proper security checks here (e.g., nonce verification)
    
    $screenName = sanitize_text_field($_POST['screenName']);
    $bets = json_decode(stripslashes($_POST['bets']), true); // Assuming bets is an array of objects
    
    if (empty($bets) || !is_array($bets)) {
        wp_send_json_error('Invalid bets data provided.');
        wp_die();
    }

    foreach ($bets as $bet) {
        // Sanitize and prepare data (Example here, adjust according to your actual data structure)
        $sport = sanitize_text_field($bet['sport']);
        $category = sanitize_text_field($bet['category']);
        $type = sanitize_text_field($bet['type']);
        $details = wp_json_encode($bet['details']); // Assuming details is an associative array
        $displayText = sanitize_text_field($bet['displayText']); // New line to sanitize displayText

        // Insert data into the wp_bet_slips table
        $insert_result = $wpdb->insert(
            $wpdb->prefix . 'bet_slips', // Using $wpdb->prefix to handle custom WordPress table prefixes
            array(
                'screen_name' => $screenName,
                'sport' => $sport,
                'category' => $category,
                'type' => $type,
                'details' => $details, // Storing details as a JSON string
                'display_text' => $displayText, // New column for displayText
            ),
            array('%s', '%s', '%s', '%s', '%s', '%s') // Data format
        );

        // Check for successful insertion
        if ($insert_result === false) {
            wp_send_json_error('Failed to insert bet slip: ' . $wpdb->last_error);
            wp_die();
        }
    }

    wp_send_json_success('Bet slips processed successfully.');
    wp_die(); // This is required to terminate immediately and return a proper response
}




function upload_bets_callback() {
    // Check if the request method is POST
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        // Get the data from the POST request
        $data = $_POST['data'];

        // Process the data (for demonstration purposes, let's just return the received data)
        echo json_encode(array('success' => true, 'data' => $data));
    }

    // Always exit to avoid further execution
    wp_die();
}

 // For not logged in users
add_action('admin_post_add_custom_bet', 'handle_custom_form_submission'); // For logged in users

if ( ! function_exists( 'twentytwentyfour_block_styles' ) ) :
	/**
	 * Register custom block styles
	 *
	 * @since Twenty Twenty-Four 1.0
	 * @return void
	 */
	function twentytwentyfour_block_styles() {

		register_block_style(
			'core/details',
			array(
				'name'         => 'arrow-icon-details',
				'label'        => __( 'Arrow icon', 'twentytwentyfour' ),
				/*
				 * Styles for the custom Arrow icon style of the Details block
				 */
				'inline_style' => '
				.is-style-arrow-icon-details {
					padding-top: var(--wp--preset--spacing--10);
					padding-bottom: var(--wp--preset--spacing--10);
				}

				.is-style-arrow-icon-details summary {
					list-style-type: "\2193\00a0\00a0\00a0";
				}

				.is-style-arrow-icon-details[open]>summary {
					list-style-type: "\2192\00a0\00a0\00a0";
				}',
			)
		);
		register_block_style(
			'core/post-terms',
			array(
				'name'         => 'pill',
				'label'        => __( 'Pill', 'twentytwentyfour' ),
				/*
				 * Styles variation for post terms
				 * https://github.com/WordPress/gutenberg/issues/24956
				 */
				'inline_style' => '
				.is-style-pill a,
				.is-style-pill span:not([class], [data-rich-text-placeholder]) {
					display: inline-block;
					background-color: var(--wp--preset--color--base-2);
					padding: 0.375rem 0.875rem;
					border-radius: var(--wp--preset--spacing--20);
				}

				.is-style-pill a:hover {
					background-color: var(--wp--preset--color--contrast-3);
				}',
			)
		);
		register_block_style(
			'core/list',
			array(
				'name'         => 'checkmark-list',
				'label'        => __( 'Checkmark', 'twentytwentyfour' ),
				/*
				 * Styles for the custom checkmark list block style
				 * https://github.com/WordPress/gutenberg/issues/51480
				 */
				'inline_style' => '
				ul.is-style-checkmark-list {
					list-style-type: "\2713";
				}

				ul.is-style-checkmark-list li {
					padding-inline-start: 1ch;
				}',
			)
		);
		register_block_style(
			'core/navigation-link',
			array(
				'name'         => 'arrow-link',
				'label'        => __( 'With arrow', 'twentytwentyfour' ),
				/*
				 * Styles for the custom arrow nav link block style
				 */
				'inline_style' => '
				.is-style-arrow-link .wp-block-navigation-item__label:after {
					content: "\2197";
					padding-inline-start: 0.25rem;
					vertical-align: middle;
					text-decoration: none;
					display: inline-block;
				}',
			)
		);
		register_block_style(
			'core/heading',
			array(
				'name'         => 'asterisk',
				'label'        => __( 'With asterisk', 'twentytwentyfour' ),
				'inline_style' => "
				.is-style-asterisk:before {
					content: '';
					width: 1.5rem;
					height: 3rem;
					background: var(--wp--preset--color--contrast-2, currentColor);
					clip-path: path('M11.93.684v8.039l5.633-5.633 1.216 1.23-5.66 5.66h8.04v1.737H13.2l5.701 5.701-1.23 1.23-5.742-5.742V21h-1.737v-8.094l-5.77 5.77-1.23-1.217 5.743-5.742H.842V9.98h8.162l-5.701-5.7 1.23-1.231 5.66 5.66V.684h1.737Z');
					display: block;
				}

				/* Hide the asterisk if the heading has no content, to avoid using empty headings to display the asterisk only, which is an A11Y issue */
				.is-style-asterisk:empty:before {
					content: none;
				}

				.is-style-asterisk:-moz-only-whitespace:before {
					content: none;
				}

				.is-style-asterisk.has-text-align-center:before {
					margin: 0 auto;
				}

				.is-style-asterisk.has-text-align-right:before {
					margin-left: auto;
				}

				.rtl .is-style-asterisk.has-text-align-left:before {
					margin-right: auto;
				}",
			)
		);
	}
endif;

add_action( 'init', 'twentytwentyfour_block_styles' );

/**
 * Enqueue block stylesheets.
 */

if ( ! function_exists( 'twentytwentyfour_block_stylesheets' ) ) :
	/**
	 * Enqueue custom block stylesheets
	 *
	 * @since Twenty Twenty-Four 1.0
	 * @return void
	 */
	function twentytwentyfour_block_stylesheets() {
		/**
		 * The wp_enqueue_block_style() function allows us to enqueue a stylesheet
		 * for a specific block. These will only get loaded when the block is rendered
		 * (both in the editor and on the front end), improving performance
		 * and reducing the amount of data requested by visitors.
		 *
		 * See https://make.wordpress.org/core/2021/12/15/using-multiple-stylesheets-per-block/ for more info.
		 */
		wp_enqueue_block_style(
			'core/button',
			array(
				'handle' => 'twentytwentyfour-button-style-outline',
				'src'    => get_parent_theme_file_uri( 'assets/css/button-outline.css' ),
				'ver'    => wp_get_theme( get_template() )->get( 'Version' ),
				'path'   => get_parent_theme_file_path( 'assets/css/button-outline.css' ),
			)
		);
	}
endif;

add_action( 'init', 'twentytwentyfour_block_stylesheets' );

/**
 * Register pattern categories.
 */

if ( ! function_exists( 'twentytwentyfour_pattern_categories' ) ) :
	/**
	 * Register pattern categories
	 *
	 * @since Twenty Twenty-Four 1.0
	 * @return void
	 */
	function twentytwentyfour_pattern_categories() {

		register_block_pattern_category(
			'page',
			array(
				'label'       => _x( 'Pages', 'Block pattern category', 'twentytwentyfour' ),
				'description' => __( 'A collection of full page layouts.', 'twentytwentyfour' ),
			)
		);
	}
endif;

add_action( 'init', 'twentytwentyfour_pattern_categories' );
