#include <Arduino.h>
#include <ESP32Lib.h>
#include <Ressources/Font6x8.h>
#include <Ressources/CodePage437_8x8.h>
#include <WiFi.h>
#include <HTTPClient.h>
#include <WiFiClientSecure.h>
#include <ArduinoJson.h>
#include <Adafruit_NeoPixel.h>
#include <vector>
#include <ctime>
#include <map>
#include <string>
#include "results.h"

#define PIN_NEO_PIXEL 15
#define NUM_PIXELS 60

//extern CRGB leds[NUM_PIXELS];
Adafruit_NeoPixel NeoPixel(NUM_PIXELS, PIN_NEO_PIXEL,NEO_GRB + NEO_KHZ800);

WiFiClient client;
HTTPClient http;

// Sound declarations
//static i2s_port_t i2s_num = I2S_NUM_1; // i2s port number


const i2s_config_t i2s_config = {
    .mode = (i2s_mode_t)(I2S_MODE_MASTER | I2S_MODE_TX),
    .sample_rate = 44100, // Note, this will be changed later
    .bits_per_sample = I2S_BITS_PER_SAMPLE_16BIT,
    .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT,
    .communication_format = (i2s_comm_format_t)(I2S_COMM_FORMAT_STAND_I2S),
    .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1, // high interrupt priority
    .dma_buf_count = 8,                       // 8 buffers
    .dma_buf_len = 1024,                      // 1K per buffer, so 8K of buffer space
    .use_apll = 0,
    .tx_desc_auto_clear = true,
    .fixed_mclk = -1};

// These are the physical wiring connections to our I2S decoder board/chip from the esp32, there are other connections
// required for the chips mentioned at the top (but not to the ESP32), please visit the page mentioned at the top for
// further information regarding these other connections.

const i2s_pin_config_t pin_config = {
    .bck_io_num = I2S_SPEAKER_SERIAL_CLOCK,
    .ws_io_num = I2S_SPEAKER_LEFT_RIGHT_CLOCK,
    .data_out_num = I2S_SPEAKER_SERIAL_DATA,
    .data_in_num = I2S_PIN_NO_CHANGE};


const char* ssid = "SDNet";
const char* password = "CapstoneProject";
const char* host = "api.sportradar.us";
const int httpsPort = 80; // Use port 443 for HTTPS
const char* apiKey = "dIf2bFcXi01nQ0rdfcPuUaJDqOMhkHXi4yaGEl13";
int displayIndex = 0; // Index of the first BetSlip to display

void updateBetSlipsFromServer();

String parseString(const String& line, const String& target);
String parseDate(const String& line, const String& target);
// Assuming the Score and Game classes are adjusted to use std::string instead of std::wstring

class Player;
class Game;

class Player {
  private:
      String fullName;
      int runs;
      int hits;
      int walks;
      int totalBases;
      int singles;
      int doubles;
      int triples;
      int homeRuns;
      int grandSlams;
      int stolenBases;
      int strikeOuts;
      int RBIs;

  public:
      // Default constructor
      Player() : fullName(""), runs(0), hits(0), walks(0), totalBases(0),
                 singles(0), doubles(0), triples(0), homeRuns(0), grandSlams(0),
                 stolenBases(0), strikeOuts(0), RBIs(0){}

      // Constructor that only sets fullName, others to 0
      Player(String fn) : fullName(fn), runs(0), hits(0), walks(0), totalBases(0),
                          singles(0), doubles(0), triples(0), homeRuns(0), grandSlams(0),
                          stolenBases(0), strikeOuts(0), RBIs(0) {}

      // Parameterized constructor
      Player(String fn, int r, int h, int w, int tb, int s, int d, int t, int hr,
             int gs, int sb, int so, int rbi) : fullName(fn), runs(r), hits(h), walks(w),
                                       totalBases(tb), singles(s), doubles(d),
                                       triples(t), homeRuns(hr), grandSlams(gs),
                                       stolenBases(sb), strikeOuts(so), RBIs(rbi) {}

      // Destructor
      ~Player() {}

      // Getters
      String getFullName() const { return fullName; }
      int getRuns() const { return runs; }
      int getHits() const { return hits; }
      int getWalks() const { return walks; }
      int getTotalBases() const { return totalBases; }
      int getSingles() const { return singles; }
      int getDoubles() const { return doubles; }
      int getTriples() const { return triples; }
      int getHomeRuns() const { return homeRuns; }
      int getGrandSlams() const { return grandSlams; }
      int getStolenBases() const { return stolenBases; }
      int getStrikeOuts() const { return strikeOuts; }
      int getRBIs() const { return RBIs; }

      // Setters
      void setFullName(String fn) { fullName = fn; }
      void setRuns(int r) { runs = r; }
      void setHits(int h) { hits = h; }
      void setWalks(int w) { walks = w; }
      void setTotalBases(int tb) { totalBases = tb; }
      void setSingles(int s) { singles = s; }
      void setDoubles(int d) { doubles = d; }
      void setTriples(int t) { triples = t; }
      void setHomeRuns(int hr) { homeRuns = hr; }
      void setGrandSlams(int gs) { grandSlams = gs; }
      void setStolenBases(int sb) { stolenBases = sb; }
      void setStrikeOuts(int so) { strikeOuts = so; }
      void setRBIs(int rbi) { RBIs = rbi; }

      bool isEqualTo(const Player& other) const {
          return this->fullName == other.fullName
                 && this->runs == other.runs
                 && this->hits == other.hits
                 && this->walks == other.walks
                 && this->totalBases == other.totalBases
                 && this->singles == other.singles
                 && this->doubles == other.doubles
                 && this->triples == other.triples
                 && this->homeRuns == other.homeRuns
                 && this->grandSlams == other.grandSlams
                 && this->stolenBases == other.stolenBases
                 && this->strikeOuts == other.strikeOuts
                 && this->RBIs == other.RBIs;
      }


};
class Game{
  private:
    String ID;
    String homeTeamName;
    String awayTeamName;
    String homeID;
    String awayID;
    int homeScore;
    int awayScore;
    std::vector<int> homeInningScores;
    std::vector<int> awayInningScores;
    std::map<String,Player> homeRoster;
    std::map<String,Player> awayRoster;
    String inningState;
    int outs;
    bool finished;

  public:

    // Default constructor
    Game() {}

    // Constructor that initializes based on Teams
    Game(const String& id)
    :ID(id) {
      homeInningScores.resize(9);
      awayInningScores.resize(9);
      finished = false;
    }

    
    // Destructor
    ~Game() {
        // Clean up resources, if any were allocated
    }

    void reset() {
        // Resetting team names and IDs
        homeTeamName = "";
        awayTeamName = "";
        homeID = "";
        awayID = "";

        // Resetting scores
        homeScore = 0;
        awayScore = 0;

        // Clearing inning scores
        homeInningScores.clear();
        awayInningScores.clear();
        homeInningScores.resize(9, 0);  // Assuming standard 9 innings, setting all to 0
        awayInningScores.resize(9, 0);

        // Clearing player rosters
        homeRoster.clear();
        awayRoster.clear();
    }

    void addPlayer(const Player& p, const String& name, const String& team_id){
      if(team_id == homeID){
        homeRoster[name] = p;
        //Serial.println("Added player not on roster to " + homeTeamName + ": " + name);
      }
      else if(team_id == awayID){
        awayRoster[name] = p;
        //Serial.println("Added player not on roster to " + awayTeamName + ": " + name);
      }
      
    }

    bool findPlayer(const String& name, const String& team_id){
      if(team_id == homeID){
        if (homeRoster.count(name) > 0){
          return true; // return 1 if player is on home team
        }
      }
      if(team_id == awayID){
        if (awayRoster.count(name) > 0){
          return true; // return 2 if player is on away team
        }
      }

      return false;
    }

    // Set Home Team's score
    void setHomeScore(const int score){
      homeScore = score;
    }

    // Set Away Team's score
    void setAwayScore(const int score){
      awayScore = score;
    }

    void setOuts(int o){
      outs = o;
    }

    int getOuts(){
      return outs;
    }

    void endGame(){
      finished = true;
    }

    bool checkProgress(){
      return finished;
    }

    // Get Home Team Score
    String getHomeScore(){
      return String(homeScore); // Convert int to string
    }

    // Get Away Team Score
    String getAwayScore(){
      return String(awayScore); // Convert int to string
    }

    void setHomeName(const String& name){homeTeamName = name;}

    void setAwayName(const String& name){awayTeamName = name;}

    String getHomeName(){
      return homeTeamName;
    }

    String getAwayName(){
      return awayTeamName;
    }

    void setHomeID(const String& ID){homeID = ID;}

    void setAwayID(const String& ID){awayID = ID;}

    String getHomeID(){
      return homeID;
    }

    String getAwayID(){
      return awayID;
    }

    String getGameId(){return ID;}

    void setInningState(bool state, int inningCount){
      if(state){
        inningState = "B" + String(inningCount);
        }
      else{
        inningState = "T" + String(inningCount);
        }
    };

    void setInningScoreHome(int inningCount, const int score) {
      if (inningCount > homeInningScores.size()) {
        homeInningScores.resize(inningCount);
      }
      homeInningScores.at(inningCount-1) = score;
    }

    void setInningScoreAway(int inningCount, const int score) {
      if (inningCount > awayInningScores.size()) {
        awayInningScores.resize(inningCount);
      }
      awayInningScores.at(inningCount-1) = score;
    }


    void updateHitCount(const String& hitterName, const bool& topBottom){
      
      if (topBottom == false){
        // away
        int totalHits = awayRoster[hitterName].getHits() + 1;
        awayRoster[hitterName].setHits(totalHits);
      }
      if (topBottom == true){
        // home
        int totalHits = homeRoster[hitterName].getHits() + 1;
        homeRoster[hitterName].setHits(totalHits);
      }
    }

    void updateSOCount(const String& pitcherName, const bool& topBottom){
      if (topBottom == false){
        // home
        int totalSOs = homeRoster[pitcherName].getStrikeOuts() + 1;
        homeRoster[pitcherName].setStrikeOuts(totalSOs);
      }
      if (topBottom == true){
        // away
        int totalSOs = awayRoster[pitcherName].getStrikeOuts() + 1;
        awayRoster[pitcherName].setStrikeOuts(totalSOs);
      }
    }

    void updateRBICount(const String& hitterName, const bool& topBottom){
      if (topBottom == false){
        // away
        int totalRBIs = awayRoster[hitterName].getRBIs() + 1;
        awayRoster[hitterName].setRBIs(totalRBIs);
      }
      if (topBottom == true){
        // home
        int totalRBIs = homeRoster[hitterName].getRBIs() + 1;
        homeRoster[hitterName].setRBIs(totalRBIs);
      }
    }
    
    void updateRunCount(const String& runnerName, const bool& topBottom) {
      if (topBottom == false){
        // away
        int totalRuns = awayRoster[runnerName].getRuns() + 1;
        awayRoster[runnerName].setRuns(totalRuns);
      }
      if (topBottom == true){
        // home
        int totalRuns = homeRoster[runnerName].getRuns() + 1;
        homeRoster[runnerName].setRuns(totalRuns);
      }
    }

    void updateSingleCount(const String& hitterName, const bool& topBottom) {
        if (topBottom == false){
        // away
        int totalSingles = awayRoster[hitterName].getSingles() + 1;
        awayRoster[hitterName].setSingles(totalSingles);
      }
      if (topBottom == true){
        // home
        int totalSingles = homeRoster[hitterName].getSingles() + 1;
        homeRoster[hitterName].setSingles(totalSingles);
      }
    }

    void updateDoubleCount(const String& hitterName, const bool& topBottom) {
        if (topBottom == false){
        // away
        int totalDoubles = awayRoster[hitterName].getDoubles() + 1;
        awayRoster[hitterName].setDoubles(totalDoubles);
      }
      if (topBottom == true){
        // home
        int totalDoubles = homeRoster[hitterName].getDoubles() + 1;
        homeRoster[hitterName].setDoubles(totalDoubles);
      }
    }

    void updateTripleCount(const String& hitterName, const bool& topBottom) {
      if (topBottom == false){
        // away
        int totalTriples = awayRoster[hitterName].getTriples() + 1;
        awayRoster[hitterName].setTriples(totalTriples);
      }
      if (topBottom == true){
        // home
        int totalTriples = homeRoster[hitterName].getTriples() + 1;
        homeRoster[hitterName].setTriples(totalTriples);
      }
    }

    void updateHRCount(const String& hitterName, const bool& topBottom) {
        if (topBottom == false){
        // away
        int totalHRs = awayRoster[hitterName].getHomeRuns() + 1;
        awayRoster[hitterName].setHomeRuns(totalHRs);
      }
      if (topBottom == true){
        // home
        int totalHRs = homeRoster[hitterName].getHomeRuns() + 1;
        homeRoster[hitterName].setHomeRuns(totalHRs);
      }
    }

    void updateStealCount(const String& runnerName, const bool& topBottom, const int& bases){
      if (topBottom == false){
        // away
        int totalSteals = awayRoster[runnerName].getStolenBases() + 1;
        awayRoster[runnerName].setStolenBases(totalSteals);
      }
      if (topBottom == true){
        // home
        int totalSteals = homeRoster[runnerName].getStolenBases() + 1;
        homeRoster[runnerName].setStolenBases(totalSteals);
      }
    }

    void updateWalkCount(const String& Name, const bool& topBottom) {  
      if (topBottom == false){
        // away
        int totalWalks = awayRoster[Name].getWalks() + 1;
        awayRoster[Name].setWalks(totalWalks);
      }
      if (topBottom == true){
        // home
        int totalWalks = homeRoster[Name].getWalks() + 1;
        homeRoster[Name].setWalks(totalWalks);
      }
    }


    void printGameDetails() {
        Serial.println("Game ID: " + ID);
        Serial.println("Current Score:");
        Serial.print("Home: " + homeTeamName + " - ");
        Serial.println(homeScore);
        Serial.print("Away: " + awayTeamName + " - ");
        Serial.println(awayScore);
    
        Serial.println("Inning Scores:");
        for (size_t i = 0; i < homeInningScores.size(); ++i) {
            Serial.print("Inning ");
            Serial.print(i + 1);
            Serial.print(": Home: ");
            Serial.print(homeInningScores[i]);
            Serial.print(" Away: ");
            Serial.println(awayInningScores[i]);
        }
    
        // Printing details of the home roster players
        Serial.println("Home Team Roster:");
        for (auto& pair : homeRoster) {
            Serial.print("Player: ");
            Serial.println(pair.first);
            printPlayerDetails(pair.second);
        }
    
        // Printing details of the away roster players
        Serial.println("Away Team Roster:");
        for (auto& pair : awayRoster) {
            Serial.print("Player: ");
            Serial.println(pair.first);
            printPlayerDetails(pair.second);
        }
    }
    
    // Helper function to print details of a player
    void printPlayerDetails(const Player& player) {
        Serial.print("Full Name: ");
        Serial.println(player.getFullName());
        Serial.print("Runs: ");
        Serial.println(player.getRuns());
        Serial.print("Hits: ");
        Serial.println(player.getHits());
        Serial.print("Walks: ");
        Serial.println(player.getWalks());
        //Serial.print("Total Bases: ");
        //Serial.println(player.getTotalBases());
        Serial.print("Singles: ");
        Serial.println(player.getSingles());
        Serial.print("Doubles: ");
        Serial.println(player.getDoubles());
        Serial.print("Triples: ");
        Serial.println(player.getTriples());
        Serial.print("Home Runs: ");
        Serial.println(player.getHomeRuns());
        //Serial.print("Grand Slams: ");
        //Serial.println(player.getGrandSlams());
        //Serial.print("Stolen Bases: ");
        //Serial.println(player.getStolenBases());
        Serial.print("Strike Outs: ");
        Serial.println(player.getStrikeOuts());
        Serial.print("RBIs: ");
        Serial.println(player.getRBIs());
    }


    String retrieveStat(const String& stat, const String& player, const String& bettingTeam) {
      Serial.println("Beginning of retrievestat");
      
      //if (homeTeamName.length() == 0) {
      //  Serial.println("homeTeamName is not initialized!");
      //}
      // if (awayTeamName.length() == 0) {
      //  Serial.println("awayTeamName is not initialized!");
      //}
      



      int playerStat;
      int statType;
      if(stat == "Score"){statType = 1;}
      if(stat == "Hits"){statType = 2;}
      if(stat == "RBIs"){statType = 3;}
      if(stat == "Strikeouts"){statType = 4;}
      if(stat == "Home Runs"){statType = 5;}
      if(stat == "Walks"){statType = 6;}


      switch(statType){
        case 1:
          if (homeTeamName == bettingTeam){
            return inningState + ", Betting Team: " + bettingTeam + " - " + String(homeScore) + ", Other Team: " + awayTeamName + " - " + String(awayScore);
          }
          else{
            return inningState + ", Betting Team: " + bettingTeam + " - " + String(awayScore) + ", Other Team: " + homeTeamName + " - " + String(homeScore);
          }
          break;
        case 2:
        
          if (homeTeamName == bettingTeam){
            playerStat = homeRoster[player].getHits();
            return player + " Hits: " + String(playerStat);
          }
          else{
            playerStat = awayRoster[player].getHits();
            return player + " Hits: " + String(playerStat);
          }
          break;
        case 3:
          if (homeTeamName == bettingTeam){
            playerStat = homeRoster[player].getRBIs();
            return player + " RBIs: " + String(playerStat);
          }
          else{
            playerStat = awayRoster[player].getRBIs();
            return player + " RBIs: " + String(playerStat);
          }
          break;
        case 4:
          if (homeTeamName == bettingTeam){
            playerStat = homeRoster[player].getStrikeOuts();
            return player + " Strikeouts: " + String(playerStat);
          }
          else{
            playerStat = awayRoster[player].getStrikeOuts();
            return player + " Strikeouts: " + String(playerStat);
          }
          break;
        case 5:
          if (homeTeamName == bettingTeam){
            playerStat = homeRoster[player].getHomeRuns();
            return player + " Home Runs: " + String(playerStat);
          }
          else{
            playerStat = awayRoster[player].getHomeRuns();
            return player + " Home Runs: " + String(playerStat);
          }
          break;
          case 6:
          if (homeTeamName == bettingTeam){
            playerStat = homeRoster[player].getWalks();
            return player + " Walks: " + String(playerStat);
          }
          else{
            playerStat = awayRoster[player].getWalks();
            return player + " Walks: " + String(playerStat);
          }
          break;
        default:
          break;
      }
      return "Invalid stat retrieval call.";
    }

};

class BetSlips {
public:
  int id;
  String displayName;
  String sport;
  String betCategory;
  String betType;
  String playerName;
  String teamName;
  String teamAbbrv;
  String overUnder;
  String hits;
  String spread;
  String RBIs;
  String Strikeouts;
  String homeRuns;
  String walks;
  String displayText;
  String datePlaced;
  double betEntry;
  double betOdds;
  double betWinnings;
  Game* gamePointer;
  int currentStatNumber;
  int desiredStat;
  int bettingScore;
  int opposingScore;
  bool betFinished;
  bool gameOver;
  bool winBet;

BetSlips() {}

BetSlips(const JsonObject& obj) {
    id = obj["id"].as<int>();
    displayName = obj["screen_name"].as<String>();
    sport = obj["sport"].as<String>();
    betCategory = obj["category"].as<String>();  // Assign "category" to betCategory
    betType = obj["type"].as<String>();          // Assign "type" to betType
    if (obj["details"].is<String>()) {
      String details = obj["details"].as<String>();
      DynamicJsonDocument detailsDoc(256);
      deserializeJson(detailsDoc, details);
      JsonObject detailsObj = detailsDoc.as<JsonObject>();
      if (betCategory == "Game Outcomes") {
        teamName = detailsObj["winningTeam"].as<String>();
        // Extracting value in parenthesis
        int startPos = teamName.indexOf('(');
        int endPos = teamName.indexOf(')');
        if (startPos != -1 && endPos != -1) {
          teamAbbrv = teamName.substring(startPos + 1, endPos);
        }
      } else if (betCategory == "Player Lines") {
        playerName = detailsObj["playerName"].as<String>();
        // Extracting value in parenthesis
        int startPos = playerName.indexOf('(');
        int endPos = playerName.indexOf(')');
        if (startPos != -1 && endPos != -1) {
          teamAbbrv = playerName.substring(startPos + 1, endPos);
        }
      }
      playerName = detailsObj["playerName"].as<String>(); // May not exist for all entries
      teamName = detailsObj["winningTeam"].as<String>();
      overUnder = detailsObj["overUnder"].as<String>();
      hits = detailsObj["hits"].as<String>();
      spread = detailsObj["spread"].as<String>();
      RBIs  = detailsObj["RBIs"].as<String>();
      Strikeouts  = detailsObj["Strikeouts"].as<String>();// May not exist for all entries
      homeRuns = detailsObj["homeRuns"].as<String>();
      walks = detailsObj["walks"].as<String>();
      // Other fields from 'details' can be extracted here similarly
    }
  displayText = obj["display_text"].as<String>();
  datePlaced = obj["created_at"].as<String>();

    int dollarSignIndex = displayText.indexOf('$');
    int toWinIndex = displayText.indexOf("to win $");

    // If indices are valid, proceed to parse the substrings
    if (dollarSignIndex != -1 && toWinIndex != -1) {
      // Extract betEntry starting after the first '$' sign
      int betEntryEndIndex = displayText.indexOf(' ', dollarSignIndex);
      String betEntryString = displayText.substring(dollarSignIndex + 1, betEntryEndIndex);
      betEntry = betEntryString.toFloat();  // Convert string to float

      // Extract betWinnings starting after "to win $"
      String betWinningsString = displayText.substring(toWinIndex + 8); // 8 is the length of "to win $"
      betWinnings = betWinningsString.toFloat();  // Convert string to float
    }
    gamePointer = NULL;
    currentStatNumber = 0;
    desiredStat = 1;
    bettingScore = 0;
    opposingScore = 0;
    betFinished = false;
    gameOver = false;
    winBet = false;
  }



  void print() {
    Serial.println("Bet Slip:");
    Serial.println("ID: " + id);
    Serial.println("Display Name: " + displayName);
    Serial.println("Sport: " + sport);
    Serial.println("Team Abbreviation: " + teamAbbrv);
    Serial.println("Spread: " + spread);
    Serial.println("Team Name: " + teamName);
    Serial.println("Player Name: " + playerName);
    Serial.println("Bet Category: " + betCategory);
    Serial.println("Bet Type: " + betType);
    Serial.println("Over/Under: " + overUnder);
    Serial.println("hits: " + hits);
    Serial.println("Strikeouts: " + Strikeouts);
    Serial.println("RBIs: " + RBIs);
    Serial.printf("Bet Entry: %f\n", betEntry);
    Serial.printf("Bet Odds: %f\n", betOdds);
    Serial.printf("Bet Winnings: %f\n", betWinnings);
  }

  
};

std::vector<BetSlips> betSlips;
std::vector<Game*> allGames = {};

int parseNumberFromString(String input) {
    String numberString = "";
    bool foundNumber = false;

    for (int i = 0; i < input.length(); i++) {
        char c = input.charAt(i);    
        if (c >= '0' && c <= '9') {
            foundNumber = true;
            numberString += c; 
        } else if (foundNumber) {
            break;
        }
    }
    int number = numberString.toInt();
    return number;

}

void updateBetSlipsFromServer() {
  // Make the HTTP GET request as done in the setup() function

  http.begin("http://gamblingscreen.com/getdata.php");
  int httpCode = http.GET();

  if (httpCode > 0) {
    String payload = http.getString();
    DynamicJsonDocument doc(2048); // Ensure the document is large enough
    deserializeJson(doc, payload);
    JsonArray array = doc.as<JsonArray>();
    Serial.println("Processing bet slips:");
    for(JsonVariant var : array) {
      JsonObject obj = var.as<JsonObject>();
      BetSlips betSlip(obj);
      Serial.println("BET ID:");
      Serial.println(betSlip.id);
      // Check if this betSlip is already in betSlips
      auto it = std::find_if(betSlips.begin(), betSlips.end(), [&betSlip](const BetSlips& b) {
        return b.id == betSlip.id;
      });

      // If the betSlip is new, add it to the vector
      if (it == betSlips.end()) {
        betSlips.push_back(betSlip);
        Serial.print("Added new bet slip with ID:");
        Serial.print(betSlip.id);
      }
    }
  } else {
    Serial.printf("HTTP GET request failed, code: %d\n", httpCode);
  }


}

DynamicJsonDocument doc(1024);
//pin configuration
const int redPin = 12;
const int greenPin = 14;
const int bluePin = 27;
const int hsyncPin = 25;
const int vsyncPin = 26;

VGA3BitI vga;
int y;
int indent;


// Function declarations

String getDate(void);
void connectWifi(const char* ssid, const char* password);
String find_game(const char* apiKey, const String& date, const String& teamAbbr);
bool update_game(Game* currGame, const char* apiKey);
Game* init_game(const String& betTeamAbbr, const char* apiKey);
String get_new_stat(Game* currGame, const String& stat, const String& player, const String& bettingTeam);

String parseString(const String& line, const String& target) {
    int pos = line.indexOf(target);
    if (pos != -1) { // Corrected comparison
        int startPos = line.indexOf('\"', pos) + 1; // Note the use of single quotes for the character literal
        int endPos = line.indexOf('\"', startPos);
        if (startPos != -1 && endPos != -1) { // Additional check for robustness
            return line.substring(startPos, endPos);
        }
    }
    return String(""); // Return an empty String instead of String(NULL)
}

String parseDate(const String& line, const String& target) {
    int pos = line.indexOf(target);
    String actualDate = "";
    String utcHour = "";
    if (pos != -1) { // Corrected comparison
        int startPos = line.indexOf('\"', pos) + 1; // Note the use of single quotes for the character literal
        int endPos = line.indexOf('T', startPos);
        if (startPos != -1 && endPos != -1) { // Additional check for robustness
            actualDate = line.substring(startPos, endPos);
        }
    }
    if (pos != -1){
      int startPos = line.indexOf('T', pos) + 1; // Note the use of single quotes for the character literal
      int endPos = line.indexOf(':', startPos);
      if (startPos != -1 && endPos != -1) { // Additional check for robustness
            utcHour = line.substring(startPos, endPos);
        }
    }
    int hour = utcHour.toInt();
    if (hour < 7){
      String day = actualDate.substring(8,10);
      int actualDay = day.toInt() - 1;
      actualDate = actualDate.substring(0,8) + String(actualDay);
    }
    return actualDate; // Return an empty String instead of String(NULL)
}

String getDate() {
    // Obtain current time
    std::time_t t = std::time(nullptr);
    
    // Convert to local time structure
    std::tm* now = std::localtime(&t);

    // Manually adjust for Pacific Standard Time (PST, UTC-8)
    // Note: Add the offset instead of subtracting to correct the underflow issue
    now->tm_hour -= 8; // Adjusting hours directly in the tm structure

    // Normalize the tm struct in case the adjustments cross day boundaries
    mktime(now); // Adjusts the time structure in case of under/overflow

    // Buffer to store the formatted date
    char buffer[11];

    // Format the date as YYYY-MM-DD
    strftime(buffer, sizeof(buffer), "%Y-%m-%d", now);

    // Return the formatted date as an Arduino String
    return String(buffer);
}

String find_game(const char* apiKey, const String& date, const String& teamAbbr){
  

  const char* host = "api.sportradar.us";
  const int httpsPort = 80; // Use port 443 for HTTPS
  String regSeasonUrl = "/mlb/trial/v7/en/games/2024/REG/schedule.xml?api_key=" + String(apiKey);
  


    Serial.print("Connecting to ");
    Serial.println(host);
    if (!client.connect(host, httpsPort)) {
        Serial.println("Connection failed!");
        return "";
    }
    client.print(String("GET ") + regSeasonUrl + " HTTP/1.1\r\n" + "Host: " + host + "\r\n" + "User-Agent: BuildFailureDetectorESP32\r\n" + "Connection: close\r\n\r\n");
    Serial.println(String("GET ") + regSeasonUrl);
    Serial.println("Request sent");
    while (client.connected()) {
        String line = client.readStringUntil('\n');
        if (line == "\r") {
            Serial.println("Headers received");
            break;
        }
    }

    String game_id = "";

    bool foundDateMatch = false;


    while(!client.available()){}

    while (client.available()) {
      String line = client.readStringUntil('\n');
      
      if (line.indexOf("game id=") != -1) {
        String rawDate = parseDate(line, "scheduled=");
        String inProgress = parseString(line, "status=");
        if(rawDate == date && inProgress != "closed"){
          foundDateMatch = true;
          game_id = parseString(line, "game id=");
        }
        else{
          foundDateMatch = false;
        }
      }
      if (foundDateMatch == true && (line.indexOf("home name=") != -1 || line.indexOf("away name=") != -1 )) {
        Serial.println(line);
        if (parseString(line, "abbr=") == teamAbbr){
          break;
        }
      }
    }
  

    if (game_id == ""){
      Serial.println("I'm an ESP32 and I fucked up getting the game ID again");
    }
  
  return game_id;

}

bool update_game(Game* currGame, const char* apiKey) {

  

  // Your function's logic here, adapted for task use

  String game_id = currGame->getGameId();
  

  const char* host = "api.sportradar.us";
  const int httpsPort = 80; // Use port 443 for HTTPS
  String url = "/mlb/trial/v7/en/games/" + String(game_id) + "/pbp.xml?api_key=" + String(apiKey);

    Serial.print("Connecting to ");
    Serial.println(host);
    if (!client.connect(host, httpsPort)) {
        Serial.println("Connection failed!");
        return false;
    }
    client.print(String("GET ") + url + " HTTP/1.1\r\n" + "Host: " + host + "\r\n" + "User-Agent: BuildFailureDetectorESP32\r\n" + "Connection: close\r\n\r\n");
    Serial.println(String("GET ") + url);
    Serial.println("Request sent");
    while (client.connected()) {
        String line = client.readStringUntil('\n');
        if (line == "\r") {
            Serial.println("Headers received");
            break;
        }
    }

  currGame->reset();


  // Parsing States:
  // 0 = Game Level
  // 1 = Total Score Level
  // 2 = Inning Level
  // 3 = Inning Score Level
  // 4 = Inning Half Level
  // 5 = At Bat Level
  // 6 = Pitch Level
  // 7 = Steal Level
  // 8 = Pitch Runners Level
  int parsingState = 0;

  // Inning State:
  // false = Top
  // true = Bottom
  bool inningState;

  String teamName;
  int runs;
  int inningCount;
  String playerName;
  String pitcherName;
  String hitterName;
  String runnerName;
  String homeID;
  String awayID;
  bool hitState;

  String temp1;
  String temp2;
  String temp3;
  int tempInt;


  bool foundBottom;
  

  while(!client.available()){}

  while (client.available()){
    String line = client.readStringUntil('\n');
    
    switch(parsingState){
      case 0: // Game Level
        //Serial.println("At Game Level!");
        if(line.indexOf("<game ") != -1){
          // Get game progress
          temp1 = parseString(line, "status=");
          
          // Set game progress
          if(temp1 == "closed" || temp1 == "canceled"){
            currGame->endGame();
          }
          parsingState = 0;
        }
        if(line.indexOf("<scoring>") != -1){
          // Go to Total Score Level
          parsingState = 1;
        }
        if(line.indexOf("<inning number") != -1){
          // Get relevant info
          temp1 = parseString(line, "number=");

          // Set relevant info
          inningCount = temp1.toInt();

          // Go to Inning Level
          parsingState = 2;
        }
        if(line.indexOf("</game>") != -1){
          // API call over
          parsingState = 100;
          break;
        }
        break;
      case 1: // Total Score Level
        //Serial.println("At Total Score Level!");
        if(line.indexOf("<home") != -1){
          // Get relevant info
          teamName = parseString(line, "abbr=");
          homeID = parseString(line, "id=");
          temp1 = parseString(line, "runs=");

          // Set relevant info
          runs = temp1.toInt();
          currGame->setHomeName(teamName);
          currGame->setHomeScore(runs);
          currGame->setHomeID(homeID);

          // Set parsingState
          parsingState = 1;
        }
        if(line.indexOf("<away") != -1){
          // Get relevant info
          teamName = parseString(line, "abbr=");
          awayID = parseString(line, "id=");
          temp1 = parseString(line, "runs=");

          // Set relevant info
          runs = temp1.toInt();
          currGame->setAwayName(teamName);
          currGame->setAwayScore(runs);
          currGame->setAwayID(awayID);

          // Set parsingState
          parsingState = 1;
        }
        if(line.indexOf("</scoring>") != -1){

          // Set parsingState
          parsingState = 2;
        }
        break;
      case 2: // Inning Level
        //Serial.println("At Inning Level!");
        if(line.indexOf("<scoring>") != -1){
        
          // Go to Inning Score Level
          parsingState = 3;
        }
        if(line.indexOf("<inning_half") != -1){
          // Get relevant info
          temp1 = parseString(line, "type=");
          

          // Set relevant info
          if(temp1 == "T"){
            foundBottom = false;
            inningState = false; 
            
          }
          else {
            inningState = true;
            foundBottom = true;
            
          }
          currGame->setInningState(inningState, inningCount);

          // Go to Inning Half Level
          parsingState = 4;
        }
        if(line.indexOf("</inning>") != -1){
          // Go to Game Level
          if(foundBottom == true){
            inningState = false;
            currGame->setInningState(inningState, inningCount);
          }
          parsingState = 0;
        }
        break;
      case 3: // Inning Score Level
        //Serial.println("At Inning Score Level!");
        if(line.indexOf("<home") != -1){
          // Get relevant info
          temp1 = parseString(line, "runs=");

          // Set relevant info
          runs = temp1.toInt();
          currGame->setInningScoreHome(inningCount, runs);

          // Set parsingState
          parsingState = 3;
        }
        if(line.indexOf("<away") != -1){
          // Get relevant info
          temp1 = parseString(line, "runs=");

          // Set relevant info
          runs = temp1.toInt();
          currGame->setInningScoreAway(inningCount, runs);

          // Set parsingState
          parsingState = 3;
        }
        if(line.indexOf("</scoring>") != -1){
          // Go back to Inning Level
          parsingState = 2;
        }
        break;
      case 4: // Inning Half Level
        if(line.indexOf("<at_bat") != -1){
          
          foundBottom = false;
          // Go to At Bat Level
          parsingState = 5;
        }
        if(line.indexOf("<lineup") != -1){
          // Get relevant info
          playerName = parseString(line, "full_name=");
          temp1 = parseString(line, "team_id=");

          //Serial.println("New player in lineup: " + playerName);
          // Set relevant info
          if(!currGame->findPlayer(playerName, temp1)){
            Player tempPlayer(playerName);
            currGame->addPlayer(tempPlayer, playerName, temp1); 
          }

          // Set parsingState
          parsingState = 4;
        }
        if(line.indexOf("</inning_half>") != -1){
          // Go back to Inning Level
          parsingState = 2;
        }
        break;
      case 5: // At Bat Level
        if(line.indexOf("<pitcher ") != -1){
          // Get relevant info
          pitcherName = parseString(line, "full_name=");

          // set parsingState
          parsingState = 5;
        }
        if(line.indexOf("<hitter ") != -1){
          // Get relevant info
          hitterName = parseString(line, "full_name=");

          // set parsingState
          parsingState = 5;
        }
        if(line.indexOf("<pitch ") != -1){
          // Get relevant info
          temp1 = parseString(line, "outcome_id=");
          temp2 = parseString(line, "status=");

          // Set relevant info
          // If the pitch is overturned it gets shown twice, but we only care about the official call, so skip overturned pitches
          if(temp2 == "overturned"){
            parsingState = 5;
            break;
          }
          if(temp1 == "aHR"){
            currGame->updateHRCount(hitterName, inningState);
            currGame->updateHitCount(hitterName, inningState);
          }
          if(temp1 == "aT" || temp1 == "aTAD4" || temp1 == "oTT3" || temp1 == "oTT4"){
            currGame->updateTripleCount(hitterName, inningState);
            currGame->updateHitCount(hitterName, inningState);
          }
          if(temp1 == "aD" || temp1 == "aDAD3" || temp1 == "aDAD4" || temp1 == "oDT2" || temp1 == "oDT3" || temp1 == "oDT4"){
            currGame->updateDoubleCount(hitterName, inningState);
            currGame->updateHitCount(hitterName, inningState);
          }
          if(temp1 == "aS" || temp1 == "aSAD2" || temp1 == "aSAD3" || temp1 == "aSAD4" || temp1 == "oST1" || temp1 == "oST2" || temp1 == "oST3" || temp1 == "oST4"){
            currGame->updateSingleCount(hitterName, inningState);
            currGame->updateHitCount(hitterName, inningState);
          }
          
          // Go to Pitch Level
          parsingState = 6;
        }
        if(line.indexOf("<lineup ") != -1){
          // Get relevant info
          playerName = parseString(line, "full_name=");
          temp1 = parseString(line, "team_id=");

          // Set relevant info
          if(!currGame->findPlayer(playerName, temp1)){
            Player tempPlayer(playerName);
            currGame->addPlayer(tempPlayer, playerName, temp1); 
          }

          // Set parsingState
          parsingState = 5;
        }
        if(line.indexOf("</at_bat>") != -1){
          // Go back to Inning Half Level
          parsingState = 4;
        }
        //if(line.indexOf("<steal ") != -1){
          // Go to Steal Level
        //  parsingState = 7;
        //}
        break;
      case 6: // Pitch Level
        if(line.indexOf("<runners>") != -1){
          // Go to Pitch Runners Level
          parsingState = 8;
        }
        if(line.indexOf("<count ") != -1){
          // Get relevant info
          temp1 = parseString(line, "balls=");
          temp2 = parseString(line, "strikes=");
          temp3 = parseString(line, "outs=");
          // Set relevant info
          // Walk count updater
          if(temp2.toInt() == 3){
            currGame->updateSOCount(pitcherName, inningState);
          }
          if(temp1.toInt() == 4){
            currGame->updateWalkCount(pitcherName, !inningState);
            currGame->updateWalkCount(hitterName, inningState);
          }
          currGame->setOuts(temp3.toInt());
          // Set parsingState
          parsingState = 6;
        }
        if(line.indexOf("</pitch>") != -1){
          // Go back to At Bat Level
          parsingState = 5;
        }
        break;
      case 7: // Steal Level
        if(line.indexOf("<runners>") != -1){
          // Go to Steal Runners Level
        }
        break;
      case 8: // Pitch Runners Level
        if(line.indexOf("<runner ") != -1){
          // Get relevant info
          runnerName = parseString(line, "full_name=");
          temp1 = parseString(line, "outcome_id=");

          // Set relevant info
          if(temp1 == "ERN" || temp1 == "eRN" || temp1 == "ERNu" || temp1 == "eRNu" || temp1 == "URN" || temp1 == "uRN" || temp1 == "SB4u" || temp1 == "SB3AD4u" || temp1 == "SB2AD4u" ){
            currGame->updateRunCount(runnerName, inningState);
            if(temp1 == "ERN" || temp1 == "ERNu" || temp1 == "URN"){
              currGame->updateRBICount(hitterName, inningState);
            }
          }
          
          // Set parsingState
          parsingState = 8;
        }
        if(line.indexOf("</runners>") != -1){
          // Go back to Pitch Level
          parsingState = 6;
        }
        break;
    }
  }

  if (parsingState != 100){
    Serial.println("Didn't reach end of API call!! NOOOOOOOOOO!!!!");
    return false;
  } 
  else{
    Serial.println("Game successully updated.");
    return true;
  }
}

Game* init_game(const String& betTeamAbbr, const char* apiKey){
  String date = getDate();
  date = "2024-04-30";
  Serial.println("Date: " + date);
  String gameId = "";
  while(gameId == ""){gameId = find_game(apiKey, date, betTeamAbbr);}
  Serial.println("Game ID: " + gameId);
  delay(1000);
  Serial.println(esp_get_free_heap_size());
  Game* currGame = new Game(gameId);
  Serial.println("Set Game Details!");
  Serial.println(esp_get_free_heap_size());
  return currGame;
}

String get_new_stat(Game* currGame, const String& stat, const String& player, const String& bettingTeam){
  String send_to_screen = currGame->retrieveStat(stat, player, bettingTeam);
  return send_to_screen;
}

void displayProgress(int desiredStat, int actualStat, int indent, int x, int y) {
  int circleSpacing = x / desiredStat; // Calculate spacing between circles  

  for (int i = 0; i <= desiredStat; i++) {
    int circleX = indent + i * circleSpacing; // Calculate x-coordinate of circle
    uint32_t circleColor;

    // Determine circle color based on actualStat
    if (i < actualStat + 1) {
      circleColor = vga.RGB(0, 255, 0); // Green color for circles with index less than actualStat
    } else {
      circleColor = vga.RGB(255, 0, 0); // Red color for other circles
    }

    // Draw circle at (circleX, y) with radius 5 and specified color
    vga.fillCircle(circleX, y, 5, circleColor);

    vga.setCursor(circleX - 3, y - 15); // Position text above the circle
    vga.setTextColor(vga.RGB(0xffffff));
    vga.print(i); // Print the number text
    //vga.setTextColor(vga.RGB(255, 255, 255)); // Reset text color to white
  }

  // Draw rectangles to connect circles
  for (int i = 0; i < desiredStat; i++) {
    int circleX = indent + i * circleSpacing; // Calculate x-coordinate of circle
    uint32_t rectangleColor;

    // Determine rectangle color based on the color of the circle to its right
    if (i < actualStat) {
      rectangleColor = vga.RGB(0, 255, 0); // Green color if the next circle is green
    } else {
      rectangleColor = vga.RGB(255, 0, 0); // Red color if the next circle is red
    }

    // Draw rectangle to connect circles
    vga.fillRect(circleX + 5, y - 2, circleSpacing - 5, 5, rectangleColor);
  }
}

void underDisplayProgress(int desiredStat, int actualStat, int indent, int x, int y) {
  int circleSpacing = x / desiredStat; // Calculate spacing between circles  

  for (int i = 0; i <= desiredStat; i++) {
    int circleX = indent + i * circleSpacing; // Calculate x-coordinate of circle
    uint32_t circleColor;

    // Determine circle color based on actualStat
    if (i < actualStat + 1) {
      circleColor = vga.RGB(255, 0, 0); // red color for circles with index less than actualStat
    } else {
      circleColor = vga.RGB(0, 255, 0); // green color for other circles
    }

    // Draw circle at (circleX, y) with radius 5 and specified color
    vga.fillCircle(circleX, y, 5, circleColor);

    vga.setCursor(circleX - 3, y - 15); // Position text above the circle
    vga.setTextColor(vga.RGB(0xffffff));
    vga.print(i); // Print the number text
    //vga.setTextColor(vga.RGB(255, 255, 255)); // Reset text color to white
  }

  // Draw rectangles to connect circles
  for (int i = 0; i < desiredStat; i++) {
    int circleX = indent + i * circleSpacing; // Calculate x-coordinate of circle
    uint32_t rectangleColor;

    // Determine rectangle color based on the color of the circle to its right
    if (i < actualStat) {
      rectangleColor = vga.RGB(255, 0, 0); // Green color if the next circle is green
    } else {
      rectangleColor = vga.RGB(0, 255, 0); // Red color if the next circle is red
    }

    // Draw rectangle to connect circles
    vga.fillRect(circleX + 5, y - 2, circleSpacing - 5, 5, rectangleColor);
  }
}

void betBox2(BetSlips &betSlip, int x, int y) {

  int boxWidth = 305;
  int boxHeight = 92;
  vga.setFont(CodePage437_8x8);
  vga.setTextColor(vga.RGB(0xffffff)); // Text color


  // Create bet box
  if (betSlip.betFinished == true && betSlip.winBet == true) {
    vga.fillRect(x, y, boxWidth, boxHeight, vga.RGB(0x00ff00)); 
    vga.fillRect(x + 1, y + 1, boxWidth - 2, boxHeight - 2, vga.RGB(0x000000)); // green if bet win
  }
else if (betSlip.betFinished == true && betSlip.winBet == false) {
    vga.fillRect(x, y, boxWidth, boxHeight, vga.RGB(0x0000ff)); // Background color
   vga.fillRect(x + 1, y + 1, boxWidth - 2, boxHeight - 2, vga.RGB(0x000000)); // red if bet loses
  }
  else {
    vga.fillRect(x, y, boxWidth, boxHeight, vga.RGB(0xffffff)); // Background color
    vga.fillRect(x + 1, y + 1, boxWidth - 2, boxHeight - 2, vga.RGB(0x000000)); // white if bet is ongoing
  }
 
  vga.setCursor(x + 7, y + 5);


  // Display Name
  const char* displayNameChar = betSlip.displayName.c_str();
  vga.print(displayNameChar);

    
  String teamNameStr = betSlip.teamName;
  String playerNameStr = betSlip.playerName;
  String betCategoryStr = betSlip.betCategory;  
  String betTypeStr = betSlip.betType;
  String overUnderStr = betSlip.overUnder;
  String strikeoutsStr = betSlip.Strikeouts;
  String RBIsStr = betSlip.RBIs;
  String hitsStr = betSlip.hits;
  String homeRunsStr = betSlip.homeRuns;
  String walksStr = betSlip.walks;
  
  String datePlaced = betSlip.datePlaced;


  int actualStat = 0;
  String player;
  int index = playerNameStr.indexOf('(');
  if (index != -1) {
    player = playerNameStr.substring(0, index-1);
  }

    
    if (betCategoryStr == "Player Lines") {
      
      vga.setCursor(x + 75, y+5);

      vga.print(playerNameStr.c_str());
      vga.setCursor(x + 75, y+15);
      vga.print(overUnderStr.c_str());
      vga.print(" ");
                  
      if (betSlip.hits != "null") {
        vga.print(hitsStr.c_str());
        vga.print(" ");
        //betSlip.desiredStat = betSlip.hits.toInt();
        betSlip.desiredStat = round(atof(betSlip.hits.c_str()));
      }
        
      if (betSlip.Strikeouts != "null") {
        vga.print(strikeoutsStr.c_str());
        vga.print(" ");
        //betSlip.desiredStat = betSlip.Strikeouts.toInt();
        betSlip.desiredStat = round(atof(betSlip.Strikeouts.c_str()));
      }

    
      if (betSlip.RBIs != "null") {
        vga.print(RBIsStr.c_str());
        vga.print(" "); 
        //betSlip.desiredStat = betSlip.RBIs.toInt();
        betSlip.desiredStat = round(atof(betSlip.RBIs.c_str()));
      }
      if (betSlip.homeRuns != "null") {
        vga.print(homeRunsStr.c_str());
        vga.print(" "); 
        //betSlip.desiredStat = betSlip.homeRuns.toInt();
        betSlip.desiredStat = round(atof(betSlip.homeRuns.c_str()));
      }
      if (betSlip.walks != "null") {
        vga.print(walksStr.c_str());
        vga.print(" "); 
        //betSlip.desiredStat = betSlip.walks.toInt();
        betSlip.desiredStat = round(atof(betSlip.walks.c_str()));
      }

      vga.print(betTypeStr.c_str());

    actualStat = 0;    


}
    else{
      vga.setCursor(x + 75, y+5);
      vga.print(teamNameStr.c_str());
      vga.setCursor(x + 75, y+15);
      vga.print(betTypeStr.c_str());
      if(betSlip.spread != "null") { // if spread
          vga.setCursor(x + 75, y+15);
          vga.print(betTypeStr.c_str());
          vga.print(" ");
          if(betSlip.spread.toInt() > 0) { // if positive spread
            vga.print("+");
            vga.print(betSlip.spread.toInt());
            //betSlip.desiredStat = betSlip.spread.toInt();
            betSlip.desiredStat = round(atof(betSlip.spread.c_str()));
          }
          else {
          vga.print(betSlip.spread.toInt()); // if negative spread
          //betSlip.desiredStat = betSlip.spread.toInt();
          betSlip.desiredStat = round(atof(betSlip.spread.c_str()));
          }
      } 
      else { // If Moneyline
        vga.setCursor(x + 75, y+15);
        vga.print(betTypeStr.c_str());
      }
    actualStat = 0;


    }
    
    vga.setCursor(x+15,y+60);
  vga.print("$");
  vga.print(betSlip.betEntry);
  vga.setCursor(x+15,y+70);
  vga.print("to win $");
  vga.print(betSlip.betWinnings);


    
   
    
    Serial.println("about to try initializing games");
      String tempTeamName = betSlip.teamAbbrv;
      Serial.println(tempTeamName);
      Game* tempGame;
      bool teamFound = false;
      if(allGames.size() == 0){
        Serial.println("allGames is empty");
          tempGame = init_game(tempTeamName, apiKey);
          while(!update_game(tempGame, apiKey)){delay(500);}
          allGames.push_back(tempGame);
          Serial.println("added frist game");
      }
      for (int gameIter = 0; gameIter < allGames.size(); gameIter++){
        //Serial.println("In first For loop");
 
        if((allGames[gameIter]->getHomeName() == tempTeamName || allGames[gameIter]->getAwayName() == tempTeamName)){
          teamFound = true;
        }
          
      }
      if(!teamFound){
        tempGame = init_game(tempTeamName, apiKey);
        while(!update_game(tempGame, apiKey)){delay(500);}
        allGames.push_back(tempGame);
      }

      for (int gameIter = 0; gameIter < allGames.size(); gameIter++){
        //Serial.println("In second for loop");
        if(betSlip.gamePointer == NULL && (allGames[gameIter]->getHomeName() == tempTeamName || allGames[gameIter]->getAwayName() == tempTeamName)){
            betSlip.gamePointer = allGames[gameIter];
            Serial.println("added gamePointer to betSlip");
        }

      }

   //Serial.println("If this is the only message, its so over :(");
   Game* currGame = betSlip.gamePointer;
   

  Serial.println("Player Name: " + playerNameStr + ", team name:" + tempTeamName); 
   String testDisplay = get_new_stat(currGame, "Score", playerNameStr, tempTeamName);

  Serial.println(testDisplay);
    


String inning;
  String bettingTeam;
int bettingTeamScore;
String opposingTeam;
int opposingTeamScore;

int inningEnd = testDisplay.indexOf(",");

// Indices for extracting "Betting Team"
int bettingTeamStart = testDisplay.indexOf("Betting Team: ") + 14; // Start after "Betting Team: "
int bettingTeamEnd = testDisplay.indexOf(" -", bettingTeamStart);

// Indices for extracting "Opposing Team"
int opposingTeamStart = testDisplay.indexOf("Other Team: ") + 12; // Start after "Other Team: "
int opposingTeamEnd = testDisplay.indexOf(" -", opposingTeamStart);

// Extract team names
inning = testDisplay.substring(0, inningEnd);
Serial.println("Inning string");
Serial.println(inning);
bettingTeam = testDisplay.substring(bettingTeamStart, bettingTeamEnd);
opposingTeam = testDisplay.substring(opposingTeamStart, opposingTeamEnd);

// Indices for extracting scores
int bettingTeamScoreStart = testDisplay.indexOf("- ", bettingTeamEnd) + 2;
int bettingTeamScoreEnd = testDisplay.indexOf(",", bettingTeamScoreStart);
int opposingTeamScoreStart = testDisplay.lastIndexOf("- ") + 2;

// Extract scores as strings
String bettingTeamScoreStr = testDisplay.substring(bettingTeamScoreStart, bettingTeamScoreEnd);
String opposingTeamScoreStr = testDisplay.substring(opposingTeamScoreStart);

// Convert score strings to integers
bettingTeamScore = bettingTeamScoreStr.toInt();
opposingTeamScore = opposingTeamScoreStr.toInt();

  vga.setCursor(x+175,y+60);
  vga.print(bettingTeam.c_str());
  vga.print(" - ");
  vga.print(bettingTeamScore);
  vga.setCursor(x+175,y+70);
  vga.print(opposingTeam.c_str());
  vga.print(" - ");
  vga.print(opposingTeamScore);

  vga.setCursor(x+250, y+60);
  vga.print(inning.c_str());
  vga.setCursor(x+250, y+70);
  String currOuts = "Outs:" + String(currGame->getOuts());
  vga.print(currOuts.c_str());



   //betSlip.gameOver = tempGame.checkProgress(); /////////////////////////////////////////////////////////////// GAME PROGRESS

  if (betCategoryStr == "Game Outcomes" && betTypeStr == "Moneyline") {

      if(bettingTeamScore > opposingTeamScore) { //Sets bar for MoneyLine if team is winning
        actualStat = actualStat;
        displayProgress(1, 1,  18, (boxWidth-30), y+45);
      }
      else {
        actualStat = 0;
        displayProgress(1, actualStat,  18, (boxWidth-30), y+45); 
      }
      
      if (bettingTeamScore > betSlip.bettingScore && opposingTeamScore == betSlip.opposingScore) {
        
          playPositiveChange(); // play positive change and update scores
          betSlip.bettingScore = bettingTeamScore;
          betSlip.opposingScore = opposingTeamScore;
      }
      else if (opposingTeamScore > betSlip.opposingScore && bettingTeamScore == betSlip.bettingScore) {
          
          playNegativeChange(); // play negative change and update scores
          betSlip.bettingScore = bettingTeamScore;
          betSlip.opposingScore = opposingTeamScore;
      }
      else {
          betSlip.bettingScore = bettingTeamScore; //update scores regardless
          betSlip.opposingScore = opposingTeamScore;
      }


      if (betSlip.gameOver == true && actualStat == 1 && betSlip.betFinished == false) { 

        playBetWin();
        betSlip.betFinished = true; // updates betslip to finish so sound is not played again

      }
      else if (betSlip.gameOver == true && actualStat == 0 && betSlip.betFinished == false) {
        
        playBetLoss();
        betSlip.betFinished = true; // updates betslip to finish so sound is not played again
      }

  }


  else if (betCategoryStr == "Player Lines" && betTypeStr == "Hits") { 
    String hitsDisplay = get_new_stat(currGame, "Hits", player, betSlip.teamAbbrv);
    actualStat = parseNumberFromString(hitsDisplay);

     if (actualStat == betSlip.desiredStat && overUnderStr == "Over" && betSlip.betFinished == false) { // If a over bet hits for the first time
        betSlip.betFinished = true;
        betSlip.winBet = true;
        betSlip.currentStatNumber = actualStat;
        playBetWin();
     }
     else if (actualStat == betSlip.desiredStat && overUnderStr  == "Under" && betSlip.betFinished == false) { // if an under bet fails for the first time
        betSlip.betFinished = true;
        betSlip.winBet = false;
        betSlip.currentStatNumber = actualStat;
        playBetLoss();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Over") { // If stat occurs and over bet (POSITIVE)
        betSlip.currentStatNumber = actualStat;
        playPositiveChange();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Under") { // If stat occurs in under bet (BAD)
        betSlip.currentStatNumber = actualStat;
        playNegativeChange();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Over") { // If game ends without over hitting
        betSlip.betFinished = true;
        playBetLoss();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Under") { // If game ends with under hitting
        betSlip.betFinished = true;
        betSlip.winBet = true;
        playBetWin();
     }
     else {
        betSlip.currentStatNumber = actualStat;
     }


  



  }
  else if (betCategoryStr == "Player Lines" && betTypeStr == "Strikeouts") { 
    String strikeoutDisplay = get_new_stat(currGame, "Strikeouts", player, betSlip.teamAbbrv);
    actualStat = parseNumberFromString(strikeoutDisplay);

    if (actualStat == betSlip.desiredStat && overUnderStr == "Over" && betSlip.betFinished == false) { // If a over bet hits for the first time
        betSlip.betFinished = true;
        betSlip.winBet = true;
        betSlip.currentStatNumber = actualStat;
        playBetWin();
     }
     else if (actualStat == betSlip.desiredStat && overUnderStr  == "Under" && betSlip.betFinished == false) { // if an under bet fails for the first time
        betSlip.betFinished = true;
        betSlip.winBet = false;
        betSlip.currentStatNumber = actualStat;
        playBetLoss();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Over") { // If stat occurs and over bet (POSITIVE)
        betSlip.currentStatNumber = actualStat;
        playPositiveChange();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Under") { // If stat occurs in under bet (BAD)
        betSlip.currentStatNumber = actualStat;
        playNegativeChange();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Over") { // If game ends without over hitting
        betSlip.betFinished = true;
        playBetLoss();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Under") { // If game ends with under hitting
        betSlip.betFinished = true;
        betSlip.winBet = true;
        playBetWin();
     }
     else {
        betSlip.currentStatNumber = actualStat;
     }

  }
  else if (betCategoryStr == "Player Lines" && betTypeStr == "RBIs") { 
    String rbiDisplay = get_new_stat(currGame, "RBIs", player, betSlip.teamAbbrv);
    actualStat = parseNumberFromString(rbiDisplay);
    
    if (actualStat == betSlip.desiredStat && overUnderStr == "Over" && betSlip.betFinished == false) { // If a over bet hits for the first time
        betSlip.betFinished = true;
        betSlip.winBet = true;
        betSlip.currentStatNumber = actualStat;
        playBetWin();
     }
     else if (actualStat == betSlip.desiredStat && overUnderStr  == "Under" && betSlip.betFinished == false) { // if an under bet fails for the first time
        betSlip.betFinished = true;
        betSlip.winBet = false;
        betSlip.currentStatNumber = actualStat;
        playBetLoss();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Over") { // If stat occurs and over bet (POSITIVE)
        betSlip.currentStatNumber = actualStat;
        playPositiveChange();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Under") { // If stat occurs in under bet (BAD)
        betSlip.currentStatNumber = actualStat;
        playNegativeChange();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Over") { // If game ends without over hitting
        betSlip.betFinished = true;
        playBetLoss();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Under") { // If game ends with under hitting
        betSlip.betFinished = true;
        betSlip.winBet = true;
        playBetWin();
     }
     else {
        betSlip.currentStatNumber = actualStat;
     }
    
  }
    else if (betCategoryStr == "Player Lines" && betTypeStr == "Home Runs") { 
    String homeRunDisplay = get_new_stat(currGame, "Home Runs", player, betSlip.teamAbbrv);
    actualStat = parseNumberFromString(homeRunDisplay);

    if (actualStat == betSlip.desiredStat && overUnderStr == "Over" && betSlip.betFinished == false) { // If a over bet hits for the first time
        betSlip.betFinished = true;
        betSlip.winBet = true;
        betSlip.currentStatNumber = actualStat;
        playBetWin();
     }
     else if (actualStat == betSlip.desiredStat && overUnderStr  == "Under" && betSlip.betFinished == false) { // if an under bet fails for the first time
        betSlip.betFinished = true;
        betSlip.winBet = false;
        betSlip.currentStatNumber = actualStat;
        playBetLoss();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Over") { // If stat occurs and over bet (POSITIVE)
        betSlip.currentStatNumber = actualStat;
        playPositiveChange();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Under") { // If stat occurs in under bet (BAD)
        betSlip.currentStatNumber = actualStat;
        playNegativeChange();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Over") { // If game ends without over hitting
        betSlip.betFinished = true;
        playBetLoss();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Under") { // If game ends with under hitting
        betSlip.betFinished = true;
        betSlip.winBet = true;
        playBetWin();
     }
     else {
        betSlip.currentStatNumber = actualStat;
     }
  }
    else if (betCategoryStr == "Player Lines" && betTypeStr == "Walks") { 
    String walksDisplay = get_new_stat(currGame, "Walks", player, betSlip.teamAbbrv);
    actualStat = parseNumberFromString(walksDisplay);

    if (actualStat == betSlip.desiredStat && overUnderStr == "Over" && betSlip.betFinished == false) { // If a over bet hits for the first time
        betSlip.betFinished = true;
        betSlip.winBet = true;
        betSlip.currentStatNumber = actualStat;
        displayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playBetWin();
     }
     else if (actualStat == betSlip.desiredStat && overUnderStr  == "Under" && betSlip.betFinished == false) { // if an under bet fails for the first time
        betSlip.betFinished = true;
        betSlip.winBet = false;
        betSlip.currentStatNumber = actualStat;
        underDisplayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playBetLoss();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Over") { // If stat occurs and over bet (POSITIVE)
        betSlip.currentStatNumber = actualStat;
        displayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playPositiveChange();
     }
     else if (actualStat > betSlip.currentStatNumber && overUnderStr == "Under") { // If stat occurs in under bet (BAD)
        betSlip.currentStatNumber = actualStat;
        underDisplayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playNegativeChange();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Over") { // If game ends without over hitting
        betSlip.betFinished = true;
        displayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playBetLoss();
     }
     else if (betSlip.gameOver == true && actualStat < betSlip.desiredStat && overUnderStr == "Under") { // If game ends with under hitting
        betSlip.betFinished = true;
        betSlip.winBet = true;
        underDisplayProgress(betSlip.desiredStat, betSlip.currentStatNumber, 18, (boxWidth-30), y+45);
        playBetWin();
     }
     else {
        betSlip.currentStatNumber = actualStat;
     }
  }
  else if (betCategoryStr == "Game Outcomes" && betTypeStr == "Spread") {
    if (betSlip.desiredStat < 0) { // If - Spread
      actualStat = bettingTeamScore - opposingTeamScore; // winning by less than spread
      Serial.println(betTypeStr);
      Serial.println(actualStat);
      if (actualStat > -1*betSlip.desiredStat) { // winning by more than spread
        actualStat = -1*betSlip.desiredStat;
      }
      else if (actualStat < 0) { // if losing
        actualStat = 0;
      }
    }
    else { // If +Spread
      actualStat = bettingTeamScore - opposingTeamScore;
      
      if (actualStat > 0) { // max bar if winning by enough
        actualStat = betSlip.desiredStat;
      }
      else if (bettingTeamScore < opposingTeamScore && actualStat*-1 < betSlip.desiredStat) { // losing by less than spread
        actualStat = opposingTeamScore - bettingTeamScore;      
    }
    else { // losing by more than spread
        actualStat = 0;

    }
  }
  }

  if (betSlip.desiredStat < 0 && betSlip.overUnder == "Over") {
      int finalNumber = betSlip.desiredStat*-1;
      displayProgress(betSlip.desiredStat*-1, actualStat, 18, (boxWidth-30), y+45);
  }
  else if (betSlip.overUnder == "Over"){
      int finalNumber = betSlip.desiredStat;
      displayProgress(betSlip.desiredStat, actualStat, 18, (boxWidth-30), y+45);
  }
  else if (betSlip.overUnder == "Under"){
      int finalNumber = betSlip.desiredStat;
      underDisplayProgress(betSlip.desiredStat, actualStat, 18, (boxWidth-30), y+45);
  }

  

}

void setup() {
Serial.begin(115200);
Serial.println(esp_get_free_heap_size());
vga.init(vga.MODE320x200, redPin, greenPin, bluePin, hsyncPin, vsyncPin);
  Serial.println(esp_get_free_heap_size());
  
    i2s_driver_install(i2s_num, &i2s_config, 0, NULL);
    i2s_set_pin(i2s_num, &pin_config);
 
  WiFi.begin(ssid, password);

  Serial.println("Connecting to WiFi");
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print(".");
  }
  Serial.println("Connected to WiFi");
  configTime(3 * 3600, 0, "pool.ntp.org", "time.nist.gov");
  delay(2000);
  
  // Define the URL to read from
  const char* url = "http://gamblingscreen.com/getdata.php";
  NeoPixel.begin();

}

void loop() {








  //static unsigned long lastUpdateTime = 0;
  //unsigned long currentTime = millis();

  //const long updateInterval = 10000; // 10 seconds

  // Update display and fetch new bets every 10 seconds
  //if (currentTime - lastUpdateTime > updateInterval) {
    //lastUpdateTime = currentTime;

    // Fetch new bets and add them to betSlips if necessary
    Serial.println("before updating from server");
    updateBetSlipsFromServer();
    Serial.println("after updating from server");

    // Clear the screen before displaying new BetSlips to avoid overwriting text
    vga.clear();

    // Display BetSlips at specified positions
    if(betSlips.size() < 2){
      Serial.println("looping if less than 2 bets");
      for (int i = 0; i < betSlips.size(); i++) {
        betBox2(betSlips[i], 2, 5);
      }
    }
    else{
    for (int i = 0; i < 2; i++) {
      Serial.println("looping if enough bets");
      int index = (displayIndex + i) % betSlips.size(); // Ensure the index wraps around
      if (!betSlips.empty()) { // Check to prevent division by zero
        if (i == 0) betBox2(betSlips[index], 2, 5);
        else if (i == 1) betBox2(betSlips[index], 2, 102);
      }
    }
    }
    delay(2000);
    for (int iter = 0; iter < allGames.size(); iter++){
      
      while (!update_game(allGames[iter], apiKey)){delay(500);}
      
    }
    delay(10000);

    // Increment the displayIndex by 2 to prepare for the next group of BetSlips
    displayIndex += 2;
    if (displayIndex >= betSlips.size()) displayIndex = 0; // Wrap around if necessary
  //}

  // Include any other loop logic here
}

